#!/usr/bin/env python
"""
_TestImpl_

Couple of test implementations for unittests

"""
from __future__ import print_function

import os
import os.path

from WMCore.Storage.Registry import registerStageOutImpl
from WMCore.Storage.StageOutImpl import StageOutImpl
from WMCore.Storage.StageOutError import StageOutFailure

class WinImpl(StageOutImpl):
    """
    _WinImpl_

    Test plugin that always returns success

    """
    def createSourceName(self, protocol, pfn):
        return "WIN!!!"


    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=False, forceMethod=False):
        print("WinImpl.createStageOutCommand(%s, %s, %s, %s)" % (sourcePFN, targetPFN, options, checksums))
        return "WIN!!!"


    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        """
        Debug a failed unittest cp command for stageOut, without re-running it,
        providing information on the environment and the certifications

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for copy command
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """
        # Build the command for debugging purposes
        copyCommand = "WinImpl.createStageOutCommand(%s, %s, %s, %s)" % (sourcePFN, targetPFN, options, checksums)

        result = "#!/bin/bash\n"
        result += """
        echo
        echo
        echo "-----------------------------------------------------------"
        echo "==========================================================="
        echo
        echo "Debugging information on failing copy command"
        echo
        echo "Current date and time: $(date +"%Y-%m-%d %H:%M:%S")"
        echo "copy command which failed: {copy_command}"
        echo "Hostname:   $(hostname -f)"
        echo "OS:  $(uname -r -s)"
        echo
        echo "PYTHON environment variables:"
        env | grep ^PYTHON
        echo
        echo "LD_* environment variables:"
        env | grep ^LD_
        echo
        echo "Source PFN: {source}"
        echo "Target PFN: {destination}"
        echo
        echo "VOMS proxy info:"
        voms-proxy-info -all
        echo "==========================================================="
        echo "-----------------------------------------------------------"
        echo
        """.format(copy_command=copyCommand, source=sourcePFN, destination=targetPFN)
        return result


    def removeFile(self, pfnToRemove):
        print("WinImpl.removeFile(%s)" % pfnToRemove)
        return "WIN!!!"


    def executeCommand(self, command):
        return 0



class FailImpl(StageOutImpl):
    """
    _FailImpl_

    Test plugin that always results in a StageOutFailure

    """

    def createSourceName(self, protocol, pfn):
        return "FAIL!!!"


    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        print("FailImpl.createStageOutCommand(%s, %s, %s, %s)" % (sourcePFN, targetPFN, options, checksums))
        return "FAIL!!!"


    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        """
        Debug a failed unittest cp command for stageOut, without re-running it,
        providing information on the environment and the certifications

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for copy command
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """
        # Build the command for debugging purposes
        copyCommand = "FailImpl.createStageOutCommand(%s, %s, %s, %s)" % (sourcePFN, targetPFN, options, checksums)

        result = self.debuggingTemplate.format(copy_command=copyCommand, source=sourcePFN, destination=targetPFN)
        return result

    def removeFile(self, pfnToRemove):
        print("FailImpl.removeFile(%s)" % pfnToRemove)
        return "FAIL!!!"


    def executeCommand(self, command):
        msg = "FailImpl returns FAIL!!!"
        raise StageOutFailure( msg)


class LocalCopyImpl(StageOutImpl):
    """
    _LocalCopyImp_

    Test plugin that copies to a local directory

    """

    def createOutputDirectory(self, targetPFN):
        """
        I guess this masks a directory?

        """

        dirName = os.path.dirname(targetPFN)

        if not os.path.isdir(dirName):
            os.makedirs(dirName)

        return


    def createSourceName(self, protocol, pfn):
        """
        This should return the same PFN
        """
        return pfn


    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        command = "cp %s %s" %(sourcePFN, sourcePFN+'2')
        return command

    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        """
        Debug a failed unittest cp command for stageOut, without re-running it,
        providing information on the environment and the certifications

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for copy command
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """
        # Build the command for debugging purposes
        copyCommand = "cp %s %s" %(sourcePFN, sourcePFN+'2')

        result = self.debuggingTemplate.format(copy_command=copyCommand, source=sourcePFN, destination=sourcePFN+'2')
        return result

    def removeFile(self, pfnToRemove):
        command = "rm  %s" %pfnToRemove
        self.executeCommand(command)
        return "WIN!!!"


registerStageOutImpl("test-win", WinImpl)
registerStageOutImpl("test-fail", FailImpl)
registerStageOutImpl("test-copy", LocalCopyImpl)



#!/usr/bin/env python
"""
_NewWorkflow_

MySQL implementation of NewWorkflow
"""

from WMCore.Database.DBFormatter import DBFormatter

class New(DBFormatter):
    """
    Create a workflow ready for subscriptions
    """
    sql = """insert into wmbs_workflow (spec, owner, name, task, type, alt_fs_close, priority)
                values (:spec, :owner, :name, :task, :type, :alt_fs_close, :priority)"""

    def execute(self, spec = None, owner = None, name = None, task = None,
                wfType = None, alt_fs_close = False, priority = None,
                conn = None, transaction = False):
        binds = {"spec": spec, "owner": owner, "name": name, "task": task,
                 "type": wfType, "alt_fs_close": int(alt_fs_close), "priority" : priority}
        self.dbi.processData(self.sql, binds, conn = conn, transaction = transaction)
        return



#!/usr/bin/env python
"""
_ConfigureState_

Populate the states tables with all known states, and set the max retries for
each state. Default to one retry.
Create the CouchDB and associated views if needed.
"""




from WMCore.Database.CMSCouch import CouchServer
from WMCore.DataStructs.WMObject import WMObject

class ConfigureState(WMObject):
    def configure(self):
        server = CouchServer(self.config.JobStateMachine.couchurl)
        dbname = 'JSM/JobHistory'
        if dbname not in server.listDatabases():
            server.createDatabase(dbname)



"""
__init__

Module for working with thread pools.
When creating a pool (use ThreadPool class), pass
the full class name (e.g. WMCore.ThreadPool.ThreadSlave).

Classes used in the threadpool overwrite the call method
of the ThreadSlave.

"""



#!/usr/bin/env python
"""
_Resubmission_

Resubmission module, this creates truncated workflows
with limited input for error recovery.
"""
from Utils.Utilities import makeList
from WMCore.Lexicon import couchurl, identifier, cmsname, dataset
from WMCore.WMSpec.StdSpecs.StdBase import StdBase
from WMCore.WMSpec.WMWorkload import WMWorkloadHelper
from WMCore.WMSpec.WMWorkloadTools import (loadSpecClassByType, validateArgumentsCreate,
                                           checkMemCore, checkEventStreams, checkTimePerEvent)


class ResubmissionWorkloadFactory(StdBase):
    """
    _ResubmissionWorkloadFactory_

    Build Resubmission workloads.
    """

    def buildWorkload(self, arguments):
        """
        _buildWorkload_

        Build a resubmission workload from a previous
        workload, it loads the workload and truncates it.
        """
        helper = WMWorkloadHelper()
        # where to find the original spec file
        originalRequestURL = "%s/%s" % (arguments['CouchURL'], arguments['CouchWorkloadDBName'])

        helper.loadSpecFromCouch(originalRequestURL, self.originalRequestName)

        helper.truncate(self.workloadName, self.initialTaskPath,
                        self.acdcServer, self.acdcDatabase,
                        self.collectionName)
        helper.ignoreOutputModules(self.ignoredOutputModules)

        # override a couple of parameters, if provided by user
        # Note that if it was provided by the user, then it's already part of the arguments too
        if "RequestPriority" in self.userArgs:
            helper.setPriority(arguments["RequestPriority"])
        if "Memory" in self.userArgs:
            helper.setMemory(arguments["Memory"])
        if "Multicore" in self.userArgs or "EventStreams" in self.userArgs:
            self.setCoresAndStreams(helper, arguments)
        if "TimePerEvent" in self.userArgs:
            helper.setTimePerEvent(arguments.get("TimePerEvent"))

        return helper

    def __call__(self, workloadName, arguments):
        StdBase.__call__(self, workloadName, arguments)
        self.originalRequestName = self.initialTaskPath.split('/')[1]
        return self.buildWorkload(arguments)

    def factoryWorkloadConstruction(self, workloadName, arguments, userArgs=None):
        """
        Resubmission factory override of the master StdBase factory.
        Builds the entire workload, with specific features to Resubmission
        requests, and also performs a sub-set of the standard validation.
        :param workloadName: string with the name of the workload
        :param arguments: dictionary with all the relevant create/assign parameters
        :param userArgs: dictionary with user specific parameters
        :return: the workload object
        """
        self.userArgs = userArgs or {}
        self.fixupArguments(arguments)
        self.validateSchema(schema=arguments)
        workload = self.__call__(workloadName, arguments)
        self.validateWorkload(workload)
        return workload

    @staticmethod
    def getWorkloadCreateArgs():
        specArgs = {"RequestType": {"default": "Resubmission"},
                    "ResubmissionCount": {"default": 1, "type": int},
                    "OriginalRequestType": {"null": False},
                    "OriginalRequestName": {"null": False},
                    "InitialTaskPath": {"optional": False,
                                        "validate": lambda x: len(x.split('/')) > 2},
                    "ACDCServer": {"default": "https://cmsweb.cern.ch/couchdb", "validate": couchurl,
                                   "attr": "acdcServer"},
                    "ACDCDatabase": {"default": "acdcserver", "validate": identifier,
                                     "attr": "acdcDatabase"},
                    "CollectionName": {"default": None, "null": True},
                    "IgnoredOutputModules": {"default": [], "type": makeList},
                    "SiteWhitelist": {"default": [], "type": makeList,
                                      "validate": lambda x: all([cmsname(y) for y in x])},
                    # it can be Chained or MC requests, so lets make it optional
                    "InputDataset": {"optional": True, "validate": dataset, "null": True},
                    ### Override StdBase parameter definition
                    "TimePerEvent": {"default": None, "type": float, "null": True, "validate": checkTimePerEvent},
                    "Memory": {"default": None, "type": float, "null": True, "validate": checkMemCore},
                    "Multicore": {"default": None, "type": int, "null": True, "validate": checkMemCore},
                    "EventStreams": {"default": None, "type": int, "null": True, "validate": checkEventStreams}
                    }

        StdBase.setDefaultArgumentsProperty(specArgs)
        return specArgs

    def fixupArguments(self, arguments):
        """
        This method will ensure that:
         * if the user provided some specific arguments, it will be passed down the chain
         * otherwise, the same argument from the original/parent workflow will be dumped
        The only arguments to be tweaked like that are:
            TimePerEvent, Memory, Multicore, EventStreams
        :param arguments: full set of arguments from creation+assignment definitions
        :return: nothing, updates are made in place
        """
        if arguments["OriginalRequestType"] == "ReReco":
            # top level arguments are already correct
            return

        specialArgs = ("TimePerEvent", "Memory", "Multicore", "EventStreams")
        argsDefinition = self.getWorkloadCreateArgs()
        for arg in specialArgs:
            if arg in self.userArgs:
                arguments[arg] = self.userArgs[arg]
                # these should not be persisted under the Step dictionary
                if arg in ("TimePerEvent", "Memory") and arguments["OriginalRequestType"] == "StepChain":
                    continue
            elif arg in ("TimePerEvent", "Memory") and arguments["OriginalRequestType"] == "StepChain":
                # there is only the top level argument, reuse it
                continue
            else:
                arguments[arg] = argsDefinition[arg]["default"]
                continue
            # now update the inner values as well
            specType = "Step" if arguments["OriginalRequestType"] == "StepChain" else "Task"
            for innerIdx in range(1, arguments.get("{}Chain".format(specType), 0) + 1):
                # innerKey is meant to be: Task1 or Step1, Task2 or Step2 ...
                innerKey = "{}{}".format(specType, innerIdx)
                # the value of either TaskName or StepName
                innerName = arguments[innerKey]["{}Name".format(specType)]
                # value to be defined inside the Task/Step
                if isinstance(self.userArgs[arg], dict):
                    arguments[innerKey][arg] = self.userArgs[arg][innerName]
                else:
                    arguments[innerKey][arg] = self.userArgs[arg]

    def setCoresAndStreams(self, workloadHelper, inputArgs):
        """
        Set helper for the Multicore and EventStreams parameters, which
        need to be dealt with in a different way depending on the parent
        spec type
        :param workloadHelper: WMWorkload object
        :param inputArgs: dictionary with the Resubmission input args
        """
        # simple and easy way to update it
        if not isinstance(inputArgs["Multicore"], dict):
            workloadHelper.setCoresAndStreams(inputArgs["Multicore"], inputArgs.get("EventStreams", 0))
        # still a simple way to update it
        elif inputArgs['OriginalRequestType'] == "TaskChain":
            workloadHelper.setCoresAndStreams(inputArgs["Multicore"], inputArgs.get("EventStreams", 0))
        # check if it's a StepChain then based on its steps mapping
        elif workloadHelper.getStepMapping():
            # map is supposed to be in the format of:
            # {'RecoPU_2021PU': ('Step1', 'cmsRun1'), 'Nano_2021PU': ('Step2', 'cmsRun2')}
            stepChainMap = workloadHelper.getStepMapping()
            # we need to create an easier map now
            coresByCmsRun = {}
            evtStreamsByCmsRun = {}
            for stepName in stepChainMap:
                cores = inputArgs["Multicore"][stepName]
                coresByCmsRun[stepChainMap[stepName][1]] = cores
                if inputArgs["EventStreams"] and isinstance(inputArgs["EventStreams"], dict):
                    streams = inputArgs["EventStreams"][stepName]
                elif inputArgs["EventStreams"]:
                    streams = inputArgs["EventStreams"]
                else:
                    streams = 0
                evtStreamsByCmsRun[stepChainMap[stepName][1]] = streams

            # Now iterate through the tasks and update it from within the steps
            for task in workloadHelper.taskIterator():
                if task.taskType() in ["Merge", "Harvesting", "Cleanup", "LogCollect"]:
                    continue
                for cmsRunName in coresByCmsRun:
                    stepHelper = task.getStepHelper(cmsRunName)
                    stepHelper.setNumberOfCores(coresByCmsRun[cmsRunName],
                                                evtStreamsByCmsRun[cmsRunName])

    def validateSchema(self, schema):
        """
        Since we skip the master validation for Resubmission specs, we better have
        some specific validation
        """
        if schema.get("ResubmissionCount", 1) > 1:
            # we cannot validate such schema
            return
        # load assignment + creation + resubmission creation args definition
        argumentDefinition = self.getWorkloadAssignArgs()
        parentSpecClass = loadSpecClassByType(schema['OriginalRequestType'])
        argumentDefinition.update(parentSpecClass.getWorkloadCreateArgs())
        argumentDefinition.update(self.getWorkloadCreateArgs())

        try:
            validateArgumentsCreate(schema, argumentDefinition)
        except Exception as ex:
            self.raiseValidationException(str(ex))

        # and some extra validation based on the parent workflow
        if schema['OriginalRequestType'] != "TaskChain":
            for param in ("TimePerEvent", "Memory"):
                if isinstance(schema.get(param), dict):
                    msg = "ACDC for parent spec of type: {} ".format(schema['OriginalRequestType'])
                    msg += "cannot have parameter: {} defined as a dictionary: {}".format(param,
                                                                                          schema[param])
                    self.raiseValidationException(msg)



#!/usr/bin/env python
"""
_NewState_

Oracle implementation for labeling a job Complete
"""


from WMCore.BossAir.MySQL.NewState import NewState as MySQLNewState

class NewState(MySQLNewState):
    """
    _NewState_

    Insert new states into bl_status
    """

    sql = """INSERT INTO bl_status (name) SELECT :name FROM dual
             WHERE NOT EXISTS (SELECT 1 FROM bl_status WHERE name = :name)"""



#!/usr/bin/env python
"""
_TestFallbackToOldBackend_

A copy of the CP implementation that's not implemented with the V2 plugins
this will let us test that the fallback works properly

"""
from __future__ import print_function

import os

from WMCore.Storage.Execute import runCommandWithOutput
from WMCore.Storage.Registry import registerStageOutImpl
from WMCore.Storage.StageOutImpl import StageOutImpl


class TestFallbackToOldBackendImpl(StageOutImpl):
    """
    _TestFallbackToOldBackendImpl_

    Implement interface for plain cp command

    """

    def createSourceName(self, protocol, pfn):
        """
        _createSourceName_

        uses pfn

        """
        return "%s" % pfn

    def createOutputDirectory(self, targetPFN):
        """
        _createOutputDirectory_

        create dir with group permission
        """
        targetdir = os.path.dirname(targetPFN)

        checkdircmd = "/bin/ls %s > /dev/null " % targetdir
        print("Check dir existence : %s" % checkdircmd)
        try:
            checkdirexitCode, output = runCommandWithOutput(checkdircmd)
        except Exception as ex:
            msg = "Warning: Exception while invoking command:\n"
            msg += "%s\n" % checkdircmd
            msg += "Exception: %s\n" % str(ex)
            msg += "Go on anyway..."
            print(msg)

        if checkdirexitCode:
            mkdircmd = "/bin/mkdir -m 775 -p %s" % targetdir
            print("=> creating the dir : %s" % mkdircmd)
            try:
                exitCode, output = runCommandWithOutput(mkdircmd)
            except Exception as ex:
                msg = "Warning: Exception while invoking command:\n"
                msg += "%s\n" % mkdircmd
                msg += "Exception: %s\n" % str(ex)
                msg += "Go on anyway..."
                print(msg)
            if exitCode:
                msg = "Warning: failed to create the dir %s with the following error:\n%s" % (targetdir, output)
                print(msg)
            else:
                print("=> dir %s correctly created" % targetdir)
        else:
            print("=> dir already exists... do nothing.")

    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        """
        _createStageOutCommand_

        Build an cp command

        """
        original_size = os.stat(sourcePFN)[6]
        print("Local File Size is: %s" % original_size)
        result = " /bin/cp "
        if options != None:
            result += " %s " % options
        result += " %s " % sourcePFN
        result += " %s " % targetPFN
        result += """
        DEST_SIZE=`/bin/ls -l %s | awk '{print $5}'`
        if [ $DEST_SIZE ] && [ '%s' -eq $DEST_SIZE ]; then
            exit 0
        else
            echo "ERROR: Size Mismatch between local and SE"
            exit 60311
        fi
        """ % (targetPFN, original_size)
        print(result)
        return result

    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        """
        Debug a failed copy command for stageOut, without re-running it,
        providing information on the environment and the certifications

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for copy command
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """
        # Build the command for debugging purposes
        copyCommand = " /bin/cp "
        if options != None:
            copyCommand += " %s " % options
        copyCommand += " %s " % sourcePFN
        copyCommand += " %s " % targetPFN

        result = self.debuggingTemplate.format(copy_command=copyCommand, source=sourcePFN, destination=targetPFN)
        return result

    def removeFile(self, pfnToRemove):
        """
        _removeFile_

        CleanUp pfn provided

        """
        command = "/bin/rm %s" % pfnToRemove
        self.executeCommand(command)


registerStageOutImpl("testFallbackToOldBackend", TestFallbackToOldBackendImpl)



#!/usr/bin/env python
"""
_JobStatusByTaskAndSite_

Oracle implementation for loading a job by scheduler status
"""


from WMCore.BossAir.MySQL.JobStatusByTaskAndSite \
     import JobStatusByTaskAndSite as MySQLJobStatusByTaskAndSite

class JobStatusByTaskAndSite(MySQLJobStatusByTaskAndSite):
    pass



"""
_New_

MySQL implementation of Users.New
"""

__all__ = []



import time
from WMCore.Database.DBFormatter import DBFormatter

class New(DBFormatter):
    sql = """INSERT IGNORE INTO wmbs_users (cert_dn, name_hn, owner, grp, group_name, role_name)
                 VALUES (:dn, :hn, :owner, :grp, :gr, :role)
          """
    sql_get_id = """SELECT id FROM wmbs_users
                    WHERE cert_dn = :dn
                    AND group_name = :gr
                    AND role_name = :role
                 """

    def execute(self, dn, hn = None, owner = None, group = None,
                group_name = '', role_name = '',
                conn = None, transaction = False):

        binds = {"dn": dn, "hn": hn, "owner": owner, "grp": group,
                 "gr": group_name, "role": role_name}

        self.dbi.processData(self.sql, binds, conn = conn,
                             transaction = transaction)

        result = self.dbi.processData( self.sql_get_id, {'dn': dn,
                                                         "gr": group_name,
                                                         "role": role_name},
                                       conn = conn, transaction = transaction )
        id = self.format(result)
        return int(id[0][0])



#!/usr/bin/env python
"""
_BulkHeritageParent_

Oracle implementation of DBSBufferFiles.HeritageLFNParent
"""

from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.BulkHeritageParent import BulkHeritageParent as MySQLBulkHeritageParent

class BulkHeritageParent(MySQLBulkHeritageParent):
    pass



from __future__ import print_function, division

CAMPAIGN_CONFIG = \
{
    "Summer16Geant4102": {
        "go": True,
        "EventsPerLumi": "x10"
    },
    "HIRun2015": {
        "go": True,
        "labels" : ["02May2016" ,"25Aug2016"],
        "overflow" : {"PRIM" : {}},
        "DDMcopies": {
            "all" : { "N" : 2 }
        },
        "custodial_override" : ["DQMIO"],
        "fractionpass": 1.0,
        "lumisize" : -1,
        "maxcopies" : 1,
        "custodial": "T1_FR_CCIN2P3_MSS",

        "NonCustodialSites": ["T2_US_Vanderbilt"],
        "SiteBlacklist": [
            "T1_US_FNAL",
            "T2_US_Purdue",
            "T2_US_Caltech",
            "T2_US_Florida",
            "T2_US_Nebraska",
            "T2_US_UCSD",
            "T2_US_Wisconsin"
        ]
    },
    "TTI2023Upg14GS": {
        "go": True,
        "EventsPerLumi": 25
    },
    "TTI2023Upg14D": {
        "go": True,
        "SiteWhitelist": ["T1_US_FNAL", "T1_ES_PIC", "T1_FR_CCIN2P3", "T1_IT_CNAF", "T1_RU_JINR", "T1_UK_RAL",
                          "T1_DE_KIT", "T2_US_MIT", "T2_US_Nebraska", "T2_US_Caltech", "T2_CH_CERN"],
        "secondaries": {"/PYTHIA6_MinBias_TuneZ2star_14TeV/TTI2023Upg14GS-FLATBS15_DES23_62_V1-v1/GEN-SIM": {},
                        "/PYTHIA6_MinBias_TuneZ2star_14TeV/TTI2023Upg14GS-DES23_62_V1-v1/GEN-SIM": {},
                        "/MinBias_TuneZ2star_14TeV-pythia6/TTI2023Upg14-DES23_62_V1-v1/GEN-SIM": {},
                        "/PYTHIA6_Tauola_TTbar_TuneZ2star_14TeV/TTI2023Upg14GS-CoolingDefect_DES23_62_V1-v1/GEN-SIM": {}
                        }
    },

    "Run2016B": {
        "go": True,
        "labels": ["03Feb2017", "15Feb2017", "22Feb2017", "18Apr2017_ver2", "18Apr2017"],
        "overflow": {"PRIM": {}},
        "fractionpass": 1.0,
        "lumisize": -1,
        "maxcopies": 1
    },
    "Run2015C_25ns": {
        "go": True,
        "labels": ["24Nov2016", "16Jan2017", "19Jan2017"],
        "DDMcopies": {
            "all": {"N": 4},
            "RECO": {"N": 1, "host": ["T1_US_FNAL_Disk"]}
        },
        "custodial_override": ["DQMIO"],
        "fractionpass": {"all": 1.0, "AOD": 1.0, "MINIAOD": 1.0, "DQMIO": 1.0, "USER": 1.0, "RAW-RECO": 1.0},
        "lumisize": -1,
        "maxcopies": 1,
        "primary_AAA": False,
        "overflow": {"PRIM": {}}
    },

    "PhaseIISpring17D": {
        "go": True,
        "toDDM": ["GEN-SIM-DIGI-RAW"],
        "lumisize": 1500,
        "maxcopies": 1,
        "fractionpass": 0.95,
        "tune": True,
        "SiteBlacklist": ["T1_US_FNAL", "T0_CH_CERN", "T2_CH_CERN_HLT", "T2_CH_CERN"],
        "secondaries": {
            "/MinBias_TuneCUETP8M1_14TeV-pythia8/PhaseIIFall16GS82-90X_upgrade2023_realistic_v1-v1/GEN-SIM": {},
            "/MinBias_TuneCUETP8M1_14TeV-pythia8/PhaseIISpring17GS-90X_upgrade2023_realistic_v9-v1/GEN-SIM": {
                "SiteWhitelist": ["T1_DE_KIT", "T2_IT_Rome", "T1_UK_RAL", "T2_ES_CIEMAT", "T2_US_Nebraska",
                                  "T2_US_Caltech", "T2_US_Wisconsin"]}}
    },

# RelVal config

  "CMSSW_9_1_0_pre1__UPSG_VBF_PU200-1492881257": {

    "SiteWhitelist": ["T1_US_FNAL"],
    "MergedLFNBase": "/store/relval",
    "Team": "relval",
    "NonCustodialGroup": "RelVal",
    "maxcopies": 1,
    "custodial": "T1_US_FNAL_MSS",
    "lumisize": -1,
    "phedex_group": "RelVal",
    "fractionpass": 0.0,
    "go": True
  },
  "CMSSW_9_1_0_pre3__fullSimPU_premix-1493915792": {

    "SiteWhitelist": [
      "T1_US_FNAL"
    ],
    "MergedLFNBase": "/store/relval",
    "Team": "relval",
    "NonCustodialGroup": "RelVal",
    "maxcopies": 1,
    "custodial": "T1_US_FNAL_MSS",
    "lumisize": -1,
    "phedex_group": "RelVal",
    "fractionpass": 0.0,
    "go": True
  },
}



#!/usr/bin/env python
"""
_NewAlgo_

Oracle implementation of DBSBuffer.NewAlgo
"""

from WMComponent.DBS3Buffer.MySQL.NewAlgo import NewAlgo as MySQLNewAlgo

class NewAlgo(MySQLNewAlgo):

    sql = """INSERT INTO dbsbuffer_algo
             (app_name, app_ver, app_fam, pset_hash, config_content, in_dbs)
             SELECT :app_name, :app_ver, :app_fam,
                    :pset_hash, :config_content, 0
             FROM DUAL
             WHERE NOT EXISTS
               ( SELECT *
                 FROM dbsbuffer_algo
                 WHERE app_name = :app_name
                 AND app_ver = :app_ver
                 AND app_fam = :app_fam
                 AND pset_hash = :pset_hash )
             """




#!/usr/bin/env python
"""
_List_

Oracle implementation of ListFileset

"""
__all__ = []



from WMCore.WMBS.MySQL.Fileset.List import List as ListFilesetMySQL

class List(ListFilesetMySQL):
    sql = ListFilesetMySQL.sql

    def format(self, result):
        """
        Some standardised formatting
        """
        out = []
        for r in result:
            for i in r.fetchall():
                res = i
                j = i[0], str(i[1]), self.truefalse(i[2]), i[3]
                out.append(j)
        return out



from __future__ import print_function

# HOWTO run:
# 1) set up WMCore
# 2) db init
#    bin/wmcore-db-init --config src/python/WMCore/examples/WorkQueue/GlobalWQConfig.py --create --modules=WMCore.WorkQueue.Database,WMCore.MsgService,WMCore.Agent.Database
# 3) run compoments:
#    bin/wmcoreD --start --config src/python/WMCore/examples/WorkQueue/MonitorServiceConfig.py
# 4) python __file__
#     -> this prints out html file, normally resulting html snippet will be used ...


from future import standard_library
standard_library.install_aliases()

import json
import random
import urllib.parse

from WMCore.WorkQueue.DataStructs.WorkQueueElement import STATES
from WMQuality.WebTools.RESTClientAPI import makeRequest


# random colour chooser (or use matplotlib colourmap)
def random_colour():
    return '#%02x%02x%02x'%(random.randint(0,255),random.randint(0,255),random.randint(0,255))


# output file name
FILE_NAME = "PlotFairyPieChart.html"
PLOT_SERVICE_URL = "http://localhost:8888"
DATA_SOURCE_URL = "http://localhost:8888/workqueue/elementsinfo"


pageTemplate = \
"""
<html>\n<body>
<p>
<img src=%(url)s/plotfairy/plot/?type=%(plotType)s&data=%(hardCodedData)s>
</p>
<br><br>
<p>
<img src=%(url)s/plotfairy/plot/?type=%(plotType)s&data=%(systemData)s>
</p>
</body>\n</html>
"""



# hard-coded data
series = [{"label": x, "colour":  random_colour(), "value": random.randint(500, 1500)} for x in STATES]
plotDefinition = \
{
    "title": "Element status",
    "series": series,
    "shadow": True,
    "percentage": False,
    "labels": False, # don't print labels when having legend
    "legend": "topright",
    "sort": "value"
}
plotDefinition["title"] = "Element status - hard-coded"
hardCodedPlotData = urllib.parse.quote(json.dumps(plotDefinition, ensure_ascii = True))




# retrieve data from the system
# URL http://localhost:8888/workqueue/elementsinfo
data, _, _, _ = makeRequest(DATA_SOURCE_URL, verb = "GET", accept = "text/json+das",
                            contentType = "application/json")
data = json.loads(data)

systemSeries = series[:]
states = {}
for s in STATES:
    states[s] = 0 # status labels tuple
for elements in data["results"]:
    states[elements["status"]] = states[elements["status"]] + 1
for s in systemSeries:
    s["value"] = states[s["label"]]

plotDefinition["title"] = "Element status - WorkQueue"
plotDefinition["series"] = systemSeries
systemData = urllib.parse.quote(json.dumps(plotDefinition, ensure_ascii = True))



# write result file / generate result html snippet
outputFile = open(FILE_NAME, 'w')
finalPage = pageTemplate % {"url": PLOT_SERVICE_URL, "plotType": "Pie",
                            "hardCodedData": hardCodedPlotData,
                            "systemData": systemData}
outputFile.write(finalPage)
outputFile.close()
print("result html page '%s' written" % FILE_NAME)



#!/usr/bin/env python
"""
_Job_

A job is owned by a jobgroup (which gives it it's workflow) and is
associated to a (set of) file(s). The job interacts with its jobgroup
to acquire/complete/fail files. A job know's it's Workflow via it's
jobgroup. A job is meaningless without a jobgroup.

A WMBS job != a job in a batch system, it's more abstract - it's the piece of
work that needs to get done.

Jobs are added to the WMBS database by their parent JobGroup, but are
responsible for updating their state (and name).
"""

from __future__ import print_function

from builtins import int, str, bytes

from WMCore.DataStructs.Job import Job as WMJob
from WMCore.DataStructs.Mask import Mask as WMMask
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMBS.File import File
from WMCore.WMBS.Mask import Mask
from WMCore.WMBS.WMBSBase import WMBSBase


class Job(WMBSBase, WMJob):
    """
    A job in WMBS
    """

    def __init__(self, name=None, files=None, id=None):
        """
        ___init___

        jobgroup object is used to determine the workflow.
        inputFiles is a list of files that the job will process.
        """
        WMBSBase.__init__(self)
        WMJob.__init__(self, name=name, files=files)

        self["id"] = id
        self["jobgroup"] = None
        self["couch_record"] = None
        self["attachments"] = {}
        self["cache_dir"] = None
        self["sandbox"] = None
        self['fwjr'] = None
        self["mask"] = Mask()
        self['custom'] = {}  # For local add-ons that we want to send to JSON

        return

    def create(self, group):
        """
        _create_

        Write the job to the database.
        """
        if self["id"] is not None:
            return

        existingTransaction = self.beginTransaction()

        self["jobgroup"] = group.id

        if self["name"] is None:
            self["name"] = makeUUID()

        jobAction = self.daofactory(classname="Jobs.New")
        jobAction.execute(jobgroup=self["jobgroup"], name=self["name"],
                          couch_record=self["couch_record"],
                          location=self["location"], cache_dir=self['cache_dir'],
                          outcome=self['outcome'], fwjr=self['fwjr'],
                          conn=self.getDBConn(),
                          transaction=self.existingTransaction())

        self.exists()

        self['mask'].save(jobID=self['id'])

        self.associateFiles()
        # self.associateWorkUnits()
        self.commitTransaction(existingTransaction)
        return

    def delete(self):
        """
        Remove a job from WMBS
        """
        deleteAction = self.daofactory(classname="Jobs.Delete")
        deleteAction.execute(id=self["id"], conn=self.getDBConn(),
                             transaction=self.existingTransaction())
        return

    def save(self, MaskAndFiles=True):
        """
        _save_

        Flush all changes that have been made to the job to the database.

        Note that this does not save the state of the job, the state has to
        be altered with the JobStateMachine, in order to preserve order and
        prevent any accidental trouble
        """
        existingTransaction = self.beginTransaction()

        saveAction = self.daofactory(classname="Jobs.Save")
        saveAction.execute(jobid=self["id"], jobgroup=self["jobgroup"],
                           name=self["name"], couch_record=self["couch_record"],
                           location=self["location"], outcome=self["outcome"],
                           cache_dir=self["cache_dir"], fwjr=self['fwjr_path'],
                           retry_count=self['retry_count'],
                           conn=self.getDBConn(),
                           transaction=self.existingTransaction())

        if MaskAndFiles:
            self['mask'].save(jobID=self['id'])
            # NOTE that file association happens multiple times (create(), save() and updates to the jobs).
            # Hence, we need to ensure the INSERT query ignores existent rows (does not insert again)
            self.associateFiles()
            # self.associateWorkUnits()

        self.commitTransaction(existingTransaction)
        return

    def exists(self):
        """
        _exists_

        Does a job exist with this name or id.
        """
        if self["id"] is not None:
            action = self.daofactory(classname="Jobs.ExistsByID")
            result = action.execute(id=self["id"], conn=self.getDBConn(),
                                    transaction=self.existingTransaction())
        else:
            action = self.daofactory(classname="Jobs.Exists")
            result = action.execute(name=self["name"], conn=self.getDBConn(),
                                    transaction=self.existingTransaction())

            if result:
                self["id"] = result

        return result

    def load(self):
        """
        _load_

        Load the job's name, id, jobgroup, state, state_time, retry_count,
        couch_record, location and outcome from the database.  Either the ID
        or the name must be set before this is called.
        """
        if self["id"] is not None:
            loadAction = self.daofactory(classname="Jobs.LoadFromID")
            results = loadAction.execute(self["id"], conn=self.getDBConn(),
                                         transaction=self.existingTransaction())
        else:
            loadAction = self.daofactory(classname="Jobs.LoadFromName")
            results = loadAction.execute(self["name"], conn=self.getDBConn(),
                                         transaction=self.existingTransaction())

        self.update(results)
        if self['mask']['FirstRun'] is not None and self['mask']['FirstRun'] == self['mask']['LastRun']:
            self['mask'].load(jobID=self['id'])
        return

    def loadData(self):
        """
        _loadData_

        Load all information about the job, including the mask and all input
        files.  Either the ID or the name must be specified before this is
        called.
        """
        existingTransaction = self.beginTransaction()

        self.load()
        self.getMask()

        fileAction = self.daofactory(classname="Jobs.LoadFiles")
        files = fileAction.execute(self["id"], conn=self.getDBConn(),
                                   transaction=self.existingTransaction())

        self["input_files"] = []
        for fileObj in files:
            newFile = File(id=fileObj["id"])
            newFile.loadData(parentage=0)
            self.addFile(newFile)

        self.commitTransaction(existingTransaction)
        return

    def getMask(self):
        """
        _getMask_

        Load the job mask from the database and return it.
        """
        self['mask'].load(jobID=self['id'])
        return self["mask"]

    def getFiles(self, type="list"):
        """
        _getFiles_

        Retrieve a list of files that are associated with the job.
        """
        if self["id"] is None:
            return WMJob.getFiles(self, type)

        existingTransaction = self.beginTransaction()
        idAction = self.daofactory(classname="Jobs.LoadFiles")
        fileIDs = idAction.execute(self["id"], conn=self.getDBConn(),
                                   transaction=self.existingTransaction())

        currentFileIDs = WMJob.getFiles(self, type="id")
        for fileID in fileIDs:
            if fileID["id"] not in currentFileIDs:
                self.loadData()
                break

        self.commitTransaction(existingTransaction)
        return WMJob.getFiles(self, type)

    def getWorkflow(self):
        """
        _getWorkflow_

        Returns the workflow, including TaskID for a job
        """

        wfAction = self.daofactory(classname='Jobs.GetWorkflowTask')
        workflow = wfAction.execute([self['id']], conn=self.getDBConn(), transaction=self.existingTransaction())

        return workflow[0]

    def associateFiles(self):
        """
        _associateFiles_

        Update the wmbs_job_assoc table with the files in the inputFiles for the
        job.
        """
        files = WMJob.getFiles(self, type="id")

        if len(files) > 0:
            addAction = self.daofactory(classname="Jobs.AddFiles")
            addAction.execute(self["id"], files, conn=self.getDBConn(),
                              transaction=self.existingTransaction())

        return

    def associateWorkUnits(self):
        """
        _associateWorkUnits_

        Add the WorkUnits that this job requires

        Returns: N/A
        """

        existsAction = self.daofactory(classname='WorkUnit.ExistsByTaskFileLumi')
        addAction = self.daofactory(classname='WorkUnit.Add')
        assocAction = self.daofactory(classname='Jobs.AddWorkUnits')

        files = WMJob.getFiles(self)
        jobMask = self['mask']

        workflow = self.getWorkflow()
        wfid = workflow['taskid']

        lumisInJob = 0
        for wmfile in files:
            fileMask = jobMask.filterRunLumisByMask(runs=wmfile['runs'])
            for runObj in fileMask:
                lumisInJob += len(runObj.lumis)

        for wmfile in files:
            fileid = wmfile['id']
            fileMask = jobMask.filterRunLumisByMask(runs=wmfile['runs'])
            for runObj in fileMask:
                run = runObj.run
                lumis = runObj.lumis
                for lumi in lumis:
                    if not existsAction.execute(taskid=wfid, fileid=fileid, run_lumi=runObj,
                                                conn=self.getDBConn(), transaction=self.existingTransaction()):
                        addAction.execute(taskid=wfid, last_unit_count=lumisInJob, fileid=fileid, run=run,
                                          lumi=lumi,
                                          conn=self.getDBConn(), transaction=self.existingTransaction())
                    assocAction.execute(jobid=self["id"], fileid=fileid, run=run, lumi=lumi,
                                        conn=self.getDBConn(), transaction=self.existingTransaction())

    def getState(self):
        """
        _getState_

        Retrieve the state that the job is currently in.
        """
        action = self.daofactory(classname="Jobs.GetState")
        state = action.execute(self["id"], conn=self.getDBConn(),
                               transaction=self.existingTransaction)

        return state

    def getOutputDBSParentLFNs(self):
        """
        _getOutputDBSParentLFNs_

        Retrieve the LFNs of the files that should be marked as the parents of
        the output of this job.  This will be the closest relative of the input
        for the job that is marked as merged.

        Note: This assumes that the input for the job will consist of files that
        are either all merged or all unmerged.  It will not work correctly if
        the input for the job consists of a mix of merged and unmerged files.
        """
        action = self.daofactory(classname="Jobs.GetOutputParentLFNs")
        parentLFNs = action.execute(self["id"], conn=self.getDBConn(),
                                    transaction=self.existingTransaction)

        return parentLFNs

    def __to_json__(self, thunker):
        """
        __to_json__

        Serialize the job object.  This will convert all Sets() to lists and
        weed out the internal data structures that don't need to be shared.
        """
        jobDict = {"name": self["name"], "mask": self["mask"],
                   "retry_count": self["retry_count"], "state": self["state"],
                   "jobgroup": self["jobgroup"], "outcome": self["outcome"],
                   "id": self["id"], "input_files": [], "cache_dir": self["cache_dir"],
                   "sandbox": self["sandbox"]}

        for inputFile in self["input_files"]:
            jobDict["input_files"].append(thunker._thunk(inputFile))
        # These attributes are added to the job object by the ChangeState
        # code, and we want to store them in couch.
        if "_id" in self:
            jobDict["_id"] = self["_id"]
        if "state_changes" in self:
            jobDict["state_changes"] = self["state_changes"]
        if "fwkjrs" in self:
            jobDict["fwkjrs"] = []
            for fwkjr in self["fwkjrs"]:
                jobDict["fwkjrs"].append(thunker._thunk(fwkjr))

        if self['custom'] != {}:
            # If we have something in the baggage, attach it.
            # Baggage must only have simple variables
            jobDict['custom'] = self['custom']

        return jobDict

    def getCache(self, refreshFlag=False):
        """
        _getCache_

        Retrieve the location of the jobCache
        """

        if not refreshFlag and self['cache_dir']:
            return self['cache_dir']

        action = self.daofactory(classname="Jobs.GetCache")
        state = action.execute(self["id"], conn=self.getDBConn(),
                               transaction=self.existingTransaction)

        return state

    def setCache(self, cacheDir):
        """
        _setCache_

        Set the location of the jobCache
        """

        action = self.daofactory(classname="Jobs.SetCache")
        state = action.execute(id=self["id"], cacheDir=cacheDir,
                               conn=self.getDBConn(),
                               transaction=self.existingTransaction)

        return state

    def completeInputFiles(self, skipFiles=None):
        """
        _completeInputFiles_

        Set the location of the jobCache
        """

        action = self.daofactory(classname="Jobs.CompleteInput")
        if skipFiles:
            state = action.execute(self["id"], lfnsToSkip={self["id"]: skipFiles},
                                   conn=self.getDBConn(),
                                   transaction=self.existingTransaction)
        else:
            state = action.execute(self["id"],
                                   conn=self.getDBConn(),
                                   transaction=self.existingTransaction)

        return state

    def failInputFiles(self):
        """
        _failInputFiles_

        Set the location of the jobCache
        """

        action = self.daofactory(classname="Jobs.FailInput")
        state = action.execute(self["id"], conn=self.getDBConn(),
                               transaction=self.existingTransaction)

        return state

    def setFWJRPath(self, fwjrPath=None):
        """
        _setFWJRPath_

        Sets the path for the fwjr in the WMBS database
        """

        if not fwjrPath:
            if 'fwjr' in self:
                fwjrPath = self['fwjr']
            else:
                return None

        action = self.daofactory(classname="Jobs.SetFWJRPath")
        state = action.execute(self['id'], fwjrPath, conn=self.getDBConn(),
                               transaction=self.existingTransaction)

        return state

    def getDataStructsJob(self):
        """
        _getDataStructsJob_

        Returns the DataStructs version of this job
        """
        job = WMJob(name=self['name'])

        # Transfer all simple keys
        for key in self:
            if isinstance(self.get(key), (str, bytes, int, float)):
                job[key] = self[key]

        for fileObj in self['input_files']:
            job['input_files'].append(fileObj.returnDataStructsFile())

        job['mask'] = WMMask()
        for key in self["mask"]:
            job["mask"][key] = self["mask"][key]

        job.baggage = self.baggage

        return job

    def loadOutputID(self):
        """
        _loadOutputID_

        Loads the output ID of the jobgroup assigned to this job
        """

        if self['id'] < 0:
            self.create()

        action = self.daofactory(classname="Jobs.LoadOutputID")
        result = action.execute(jobID=self['id'], conn=self.getDBConn(),
                                transaction=self.existingTransaction)

        return result



from WMCore.Database.CMSCouch import Document
from WMCore.Services.RequestDB.RequestDBReader import RequestDBReader


class RequestDBWriter(RequestDBReader):
    def __init__(self, couchURL, couchapp="ReqMgr"):
        # set the connection for local couchDB call
        # inherited from WMStatsReader
        self._commonInit(couchURL, couchapp)

    def insertGenericRequest(self, doc):

        doc = Document(doc["RequestName"], doc)
        result = self.couchDB.commitOne(doc)
        self.updateRequestStatus(doc["RequestName"], "new")
        return result

    def updateRequestStatus(self, request, status, dn=None):
        status = {"RequestStatus": status}
        if dn:
            status["DN"] = dn
        return self.couchDB.updateDocument(request, self.couchapp, "updaterequest",
                                           status)

    def updateRequestStats(self, request, stats):
        """
        This function is only available ReqMgr couch app currently (not T0)
        WQ specific function
        param: stats: dict of {'total_jobs': 0, 'input_lumis': 0,
                               'input_events': 0, 'input_num_files': 0}
        """
        return self.couchDB.updateDocument(request, self.couchapp, "totalstats",
                                           fields=stats)

    def updateRequestProperty(self, request, propDict, dn=None):
        if dn:
            propDict["DN"] = dn
        return self.couchDB.updateDocument(request, self.couchapp, "updaterequest",
                                           propDict, useBody=True)



from builtins import str, bytes
from future.utils import viewitems

import json
import logging

from WMCore.Services.Service import Service


class WMStatsServer(Service):
    """
    API to query wmstats server: mostly against acive datacache

    """

    def __init__(self, url, header=None, logger=None):
        """
        responseType will be either xml or json
        """

        httpDict = {}
        header = header or {}
        # url is end point
        httpDict['endpoint'] = "%s/data" % url
        httpDict['logger'] = logger if logger else logging.getLogger()

        # cherrypy converts request.body to params when content type is set
        # application/x-www-form-urlencodeds
        httpDict.setdefault("content_type", 'application/json')
        httpDict.setdefault('cacheduration', 0)
        httpDict.setdefault("accept_type", "application/json")
        httpDict.update(header)
        self.encoder = json.dumps
        Service.__init__(self, httpDict)

    def _getResult(self, callname, clearCache=True, args=None, verb="GET",
                   encoder=json.loads, decoder=json.loads, contentType=None):
        """
        _getResult_
        """
        cfile = callname.replace("/", "_")
        if clearCache:
            self.clearCache(cfile, args, verb)

        f = self.refreshCache(cfile, callname, args, encoder=encoder,
                              verb=verb, contentType=contentType)
        result = f.read()
        f.close()

        if result and decoder:
            result = decoder(result)

        return result['result']

    def _createQuery(self, queryDict):
        """
        _createQuery
        :param queryDict: is the format of {name: values} fair. value can be sting, int or list
        :type queryDict: dict
        :returns: url query string
        """
        args = ""
        for name, values in viewitems(queryDict):
            if isinstance(values, (str, bytes, int)):
                values = [values]
            for val in values:
                args += '%s=%s&' % (name, val)

        return args.rstrip('&')

    def getGlobalLocks(self):
        """
        _getGlobalLocks_

        A public method to return the global locks from WMStatServer.
        :returns: A list of datasets
        """
        callname = 'globallocks'
        return self._getResult(callname, clearCache=True, verb="GET")

    def getActiveData(self):

        """
        _getActiveData_

        :returns:
        """

        callname = 'requestcache'
        return self._getResult(callname, verb="GET")[0]

    def getFilteredActiveData(self, inputCondition, outputMask):
        """
        _getFilteredActiveData_

        :param inputCondition: dict of Condition
        :type inputCondition: dict
        :param outputMask: list of output mask
        :type outputMask: list
        :returns:  list of dict or which passes the input condition and only result on outputMask
        """
        inputCondition.update({'mask': outputMask})
        query = self._createQuery(inputCondition)
        callname = 'filtered_requests?%s' % query
        return self._getResult(callname, verb="GET")

    def getChildParentDatasetMap(self, requestType="StepChain", parentageResolved=False):
        """

        :param requestType: specify the type of requests you want find the parentag
        :param dataset: child dataset
        :param includeInputDataset:
        :return: dict of {child_dataset_name: parent_dataset_name}
        """
        queryFilter = {"RequestType": requestType, "ParentageResolved": parentageResolved}
        mask = ["ChainParentageMap"]

        results = self.getFilteredActiveData(queryFilter, mask)
        childParentMap = {}
        for info in results:
            if info["ChainParentageMap"]:
                for childParentDS in info["ChainParentageMap"]:
                    if childParentDS["ParentDset"]:
                        for childDS in childParentDS["ChildDsets"]:
                            childParentMap[childDS] = childParentDS["ParentDset"]
        return childParentMap

    def getProtectedLFNs(self):
        """
        A method to be used for fetching a list of all protected lfns from WMStatServer
        :returns: a unique list of protected LFNs
        """
        callname = 'protectedlfns'
        return list(set(self._getResult(callname, verb="GET")))



#!/usr/bin/env python
"""
_InsertOutput_

MySQL implementation of Workflow.InsertOutput
"""

from WMCore.Database.DBFormatter import DBFormatter

class InsertOutput(DBFormatter):
    sql = """INSERT INTO wmbs_workflow_output (workflow_id, output_identifier,
                                               output_fileset, merged_output_fileset)
               SELECT :workflow AS workflow_id, :output AS output_identifier,
                 :fileset AS output_fileset, :mfileset AS merged_output_fileset
                 FROM DUAL WHERE NOT EXISTS
               (SELECT workflow_id FROM wmbs_workflow_output
                 WHERE :workflow = workflow_id AND
                       :output = output_identifier AND
                       :fileset = output_fileset)"""

    def execute(self, workflowID, outputIdentifier, filesetID, mergedFilesetID,
                conn = None, transaction = False):
        binds = {"workflow": workflowID, "output": outputIdentifier,
                 "fileset": filesetID, "mfileset": mergedFilesetID}
        self.dbi.processData(self.sql, binds, conn = conn,
                             transaction = transaction)
        return



#!/usr/bin/env python
"""
Oracle implementation of Workflow.GetUnfinishedWorkflows
"""

from WMCore.WMBS.MySQL.Workflow.GetUnfinishedWorkflows import \
    GetUnfinishedWorkflows as MySQLGetUnfinishedWorkflows


class GetUnfinishedWorkflows(MySQLGetUnfinishedWorkflows):
    pass



"""
_WorkUnit.Add_

MySQL implementation of WorkUnit.GetByID
"""

from __future__ import absolute_import, division, print_function

from WMCore.DataStructs.Run import Run
from WMCore.Database.DBFormatter import DBFormatter


class GetByID(DBFormatter):
    """
    _WorkUnit.GetByID_

    MySQL implementation of WorkUnit.GetByID
    """

    sql = ('SELECT wu.id as id, wu.taskid as taskid,  wu.retry_count as retry_count,'
           ' wu.last_unit_count as last_unit_count, wu.last_submit_time as last_submit_time, wu.status as status,'
           ' assoc.firstevent as firstevent, assoc.lastevent as lastevent,'
           ' assoc.fileid as fileid, assoc.run as run,  assoc.lumi as lumi'
           ' FROM wmbs_workunit wu'
           ' INNER JOIN wmbs_frl_workunit_assoc assoc'
           ' ON wu.id = assoc.workunit'
           ' WHERE wu.id = :wuid')

    intFields = ['id', 'taskid', 'retry_count', 'last_unit_count', 'last_submit_time', 'status',
                 'firstevent', 'lastevent', 'fileid']

    def formatOneDict(self, result):
        """
        _formatOneDict_

        Return the row as a dict
        """

        formattedResult = DBFormatter.formatDict(self, result)[0]
        return workUnitFormatter(formattedResult)

    def execute(self, wuid=None, conn=None, transaction=False):
        """
        Execute the lookup by ID
        """

        binds = {'wuid': wuid}
        result = self.dbi.processData(self.sql, binds, conn=conn, transaction=transaction)
        return self.formatOneDict(result)


def workUnitFormatter(formattedResult):
    """
    Args:
        formattedResult: The one row that comes out of DBFormatter.formatDict

    Returns:
        A dictionary with integers correctly cast and the run/lumi as a Run object
    """

    intFields = ['id', 'taskid', 'retry_count', 'last_unit_count', 'last_submit_time', 'status',
                 'firstevent', 'lastevent', 'fileid']

    for field in intFields:
        if field in formattedResult:
            formattedResult[field] = int(formattedResult[field])

    # Run/lumi handled specially
    formattedResult['run_lumi'] = Run(int(formattedResult['run']), *[int(formattedResult['lumi'])])
    del formattedResult['run']
    del formattedResult['lumi']

    return formattedResult



from WMCore.Database.DBFormatter import DBFormatter

class DefaultFormatter(DBFormatter):
    """
    change default format as dict format for monitoring
    """

    def format(self, result):
        return self.formatDict(result)



#!/usr/bin/env python
"""
_InsertThreshold_

This module inserts thresholds for the given sites for MySQL.  If the thresholds
have already been inserted this will modify them.
"""




from WMCore.Database.DBFormatter import DBFormatter

class InsertThreshold(DBFormatter):
    """
    _InsertThreshold_

    This module inserts thresholds for the given sites for MySQL.  If the thresholds
    have already been inserted this will modify them.
    """
    selSQL = """SELECT * FROM rc_threshold WHERE
                  site_id = (SELECT id FROM wmbs_location WHERE site_name = :sitename) AND
                  sub_type_id = (SELECT id FROM wmbs_sub_types WHERE name  = :tasktype)"""

    updSQL = """UPDATE rc_threshold SET max_slots = :maxslots, pending_slots = :pendslots
                WHERE site_id = (SELECT id FROM wmbs_location WHERE site_name = :sitename) AND
                      sub_type_id = (SELECT id FROM wmbs_sub_types WHERE name  = :tasktype)"""

    addSQL = """INSERT INTO rc_threshold (site_id, sub_type_id, max_slots, pending_slots) VALUES (
                 (SELECT id FROM wmbs_location WHERE site_name = :sitename),
                 (SELECT id FROM wmbs_sub_types WHERE name  = :tasktype),
                 :maxslots, :pendslots)"""

    def execute(self, siteName, taskType, maxSlots, pendingSlots, conn = None,
                transaction = False):
        binds = {"sitename": siteName, "tasktype": taskType,
                 "maxslots": maxSlots, "pendslots": pendingSlots}
        result = self.dbi.processData(self.selSQL, {"sitename": siteName, "tasktype": taskType},
                                      conn = conn, transaction = transaction)
        result = self.formatDict(result)

        if len(result) == 0:
            if maxSlots is None:
                binds['maxslots'] = pendingSlots
            if pendingSlots is None:
                binds['pendslots'] = maxSlots
            self.dbi.processData(self.addSQL, binds, conn = conn, transaction = transaction)
        else:
            if pendingSlots is None:
                binds['pendslots'] = result[0]['pending_slots']
            if maxSlots is None:
                binds['maxslots'] = result[0]['max_slots']
            self.dbi.processData(self.updSQL, binds, conn = conn, transaction = transaction)

        return



"""
Oracle implementation of GetLocationFile
"""
from WMCore.WMBS.MySQL.Files.GetLocation import GetLocation \
     as GetLocationFileMySQL

class GetLocation(GetLocationFileMySQL):
    """
    _GetLocation_

    Oracle specific: file is reserved word

    """
    pass



#!/usr/bin/env python
"""
_SetLocationForWorkQueue_

Oracle implementation of Files.SetLocationForWorkQueue

For WorkQueue only
"""

from WMCore.WMBS.MySQL.Files.SetLocationForWorkQueue import SetLocationForWorkQueue as MySQLSetLocationForWorkQueue


class SetLocationForWorkQueue(MySQLSetLocationForWorkQueue):
    """
    Oracle version


    """

    insertSQL = """INSERT /*+ IGNORE_ROW_ON_DUPKEY_INDEX (wmbs_file_location (fileid, pnn)) */ 
                     INTO wmbs_file_location (fileid, pnn)
                   SELECT wfd.id, wpnn.id FROM wmbs_file_details wfd, wmbs_pnns wpnn
                   WHERE wfd.lfn = :lfn AND wpnn.pnn = :location
                """



#!/usr/bin/env python
"""
_ListByStateAndLocation_

Oracle implementation of ListByStateAndLocation

Created on Feb 21, 2013

@author: dballest
"""

from WMCore.WMBS.MySQL.Jobs.ListByStateAndLocation import ListByStateAndLocation as MySQLListByStateAndLocation

class ListByStateAndLocation(MySQLListByStateAndLocation):
    pass



#!/usr/bin/env python
"""
_Delete_

Oracle implementation of Subscription.Delete
"""

__all__ = []



from WMCore.WMBS.MySQL.Subscriptions.Delete import Delete as DeleteMySQL

class Delete(DeleteMySQL):
    sql = DeleteMySQL.sql



#!/usr/bin/env python
"""
_List_

Oracle implementation of Subscription.List
"""

__all__ = []



from WMCore.WMBS.MySQL.Subscriptions.List import List as ListMySQL

class List(ListMySQL):
    sql =  ListMySQL.sql



#!/usr/bin/env python
"""
_GetNumberOfJobsPerWorkflow_

Oracle implementation of Jobs.GetNumberOfJobsPerWorkflow
"""

__all__ = []



import logging

from WMCore.WMBS.MySQL.Jobs.GetNumberOfJobsPerWorkflow import \
     GetNumberOfJobsPerWorkflow as MySQLGetNumberOfJobsPerWorkflow


class GetNumberOfJobsPerWorkflow(MySQLGetNumberOfJobsPerWorkflow):
    """
    Same as MySQL

    """



#!/usr/bin/env python
"""
_GetLocationsForJobs_

Oracle implementation of JobGroup.GetLocationsForJobs
"""

from WMCore.WMBS.MySQL.JobGroup.GetLocationsForJobs import GetLocationsForJobs as MySQLGetLocationsForJobs

class GetLocationsForJobs(MySQLGetLocationsForJobs):
    pass



"""
_GetSemiFinishedTasks_

MySQL implementation of Subscriptions.GetSemiFinishedTasks

Created on Nov 7, 2012

@author: dballest
"""

from time import time

from WMCore.Database.DBFormatter import DBFormatter

class GetSemiFinishedTasks(DBFormatter):
    """
    _GetSemiFinishedTasks_

    In some particular cases (e.g. Repack) the workflows
    can't be marked as injected for some time which prevents
    the GetAndMarkNewFinishedSubscriptions DAO from picking actually finished
    subscriptions. This DAO is a more relaxed version of the former, but it doesn't mark
    anything and returns tasks instead of subscriptions.

    The relaxation is present in the following checks:

    - Not injected workflows are admitted
    - Doesn't check for files which are shared with other workflows

    It also allows a pattern to filter in the workflow name
    """

    sql = """SELECT complete_subscriptions.name, complete_subscriptions.task,
                    complete_subscriptions.spec
             FROM
                (SELECT wmbs_subscription.id AS sub_id,
                        wmbs_workflow.task AS task,
                        wmbs_workflow.name AS name,
                        wmbs_workflow.id AS workflow,
                        wmbs_workflow.spec AS spec
                 FROM wmbs_subscription
                 INNER JOIN wmbs_fileset ON
                     wmbs_fileset.id = wmbs_subscription.fileset AND
                     wmbs_fileset.open = 0
                 INNER JOIN wmbs_workflow ON
                     wmbs_workflow.id = wmbs_subscription.workflow AND
                     wmbs_workflow.name LIKE :pattern
                 LEFT OUTER JOIN wmbs_sub_files_available ON
                     wmbs_sub_files_available.subscription = wmbs_subscription.id
                 LEFT OUTER JOIN wmbs_sub_files_acquired ON
                     wmbs_sub_files_acquired.subscription = wmbs_subscription.id
                 LEFT OUTER JOIN wmbs_jobgroup ON
                     wmbs_jobgroup.subscription = wmbs_subscription.id
                 LEFT OUTER JOIN wmbs_job ON
                     wmbs_job.jobgroup = wmbs_jobgroup.id AND
                     wmbs_job.state_time > :maxTime AND
                     wmbs_job.state != %d
                 WHERE wmbs_subscription.finished = 0
                 GROUP BY wmbs_workflow.task,
                          wmbs_workflow.name,
                          wmbs_workflow.spec,
                          wmbs_workflow.id,
                          wmbs_subscription.id
                 HAVING COUNT(wmbs_sub_files_available.subscription) = 0
                 AND COUNT(wmbs_sub_files_acquired.subscription) = 0
                 AND COUNT(wmbs_job.id) = 0 ) complete_subscriptions
             RIGHT OUTER JOIN wmbs_subscription ON
                 wmbs_subscription.workflow = complete_subscriptions.workflow
             GROUP BY complete_subscriptions.name, complete_subscriptions.task,
                      complete_subscriptions.spec, complete_subscriptions.workflow
             HAVING COUNT(complete_subscriptions.sub_id) = COUNT(wmbs_subscription.id)
          """

    def execute(self, state, pattern = '%Repack%', timeOut = None, conn = None, transaction = False):

        currentTime = int(time())
        if timeOut == None:
            timeOut = currentTime

        binds = {'maxTime'   : currentTime - timeOut,
                 'pattern'   : pattern}

        result = self.dbi.processData(self.sql % state,
                                      binds, conn = conn,
                                      transaction = transaction)
        return self.formatDict(result)



#!/usr/bin/env python
"""
_SetStatus_

Oracle implementation of DBSBufferFiles.SetStatus
"""




from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.SetStatus import SetStatus as MySQLSetStatus

class SetStatus(MySQLSetStatus):
    """
    _SetStatus_

    Oracle implementation of SetStatus
    """
    pass



"""
_UpdateWorker_

Oracle implementation of UpdateWorker
"""

__all__ = []



from WMCore.Agent.Database.MySQL.UpdateWorker import UpdateWorker \
     as UpdateWorkerMySQL

class UpdateWorker(UpdateWorkerMySQL):
    pass



#!/usr/bin/env python
"""
_IncrementRetry_

Oracle implementation of Jobs.IncrementRetry
"""




from WMCore.WMBS.MySQL.Jobs.IncrementRetry import IncrementRetry as IncrementRetryMySQL

class IncrementRetry(IncrementRetryMySQL):
    pass



#!/usr/bin/env python
"""
_RESTFormatter_

A basic REST formatter. The formatter takes the data from the API call, turns it into the
appropriate format and sets the CherryPy header appropriately.

Could add YAML via http://pyyaml.org/
"""

from builtins import str

import json
from types import GeneratorType

from cherrypy import response, HTTPError, request

from WMCore.WebTools.Page import TemplatedPage, _setCherryPyHeaders
from WMCore.Wrappers.JsonWrapper.JSONThunker import JSONThunker

from Utils.Utilities import decodeBytesToUnicodeConditional
from Utils.PythonVersion import PY3

class RESTFormatter(TemplatedPage):
    def __init__(self, config):
        self.supporttypes = {
                    'text/json': self.json,
                    'application/xml': self.xml,
                   'application/atom+xml': self.atom,
                   'text/x-json': self.json,
                   'application/json': self.json,
                   'text/html': self.to_string,
                   'text/plain': self.to_string,
                   '*/*': self.to_string}

        TemplatedPage.__init__(self, config)

    def genstreamer(self, data):
        yield "[\n"
        firstItem  = True
        for item in data:
            if not firstItem:
                yield "\n,\n"
            else:
                firstItem = False
            yield json.dumps(item)
        yield "\n]"

    def json(self, data):
        if isinstance(data, GeneratorType):
            out = ''.join([r for r in self.genstreamer(data)])
            return out
        thunker = JSONThunker()
        data = thunker.thunk(data)
        return json.dumps(data)

    def xml(self, data):
        if isinstance(data, GeneratorType):
            data = [i for i in data]
        return self.templatepage('XML', data = data,
                                config = self.config,
                                path = request.path_info)

    def atom(self, data):
        if isinstance(data, GeneratorType):
            data = [i for i in data]
        return self.templatepage('Atom', data = data,
                                config = self.config,
                                path = request.path_info)

    def to_string(self, data):
        """
        Since json.dumps returns unicode in py3 and bytes in py2 (it behaves
        returning "native str" in both versions), then we do the same with all
        the data that is not GeneratorType, dict nor list by calling str(data)
        Moreover, we need to properly decode bytes.

        :returns: "native str" (unicode in py3, bytes in py2)
        """
        if isinstance(data, GeneratorType):
            return self.json(data)
        if isinstance(data, dict) or isinstance(data, list):
            return json.dumps(data)
        if not isinstance(data, bytes):
            return str(data)
        return decodeBytesToUnicodeConditional(data, condition=PY3)

    def format(self, data, datatype, expires):
        response_data = ''
        func = self.supporttypes[datatype]
        if datatype not in self.supporttypes:
            response.status = 406
            expires=0
            response_data = self.supporttypes['text/plain']({'exception': 406,
                                                'type': 'HTTPError',
            'message': '%s is not supported. Valid accept headers are: %s' %\
                    (datatype, list(self.supporttypes))})

        try:
            response_data = self.supporttypes[datatype](data)
        except HTTPError as h:
            # This won't be triggered with a default formatter, but could be by a subclass
            response.status = h.args[0]
            expires=0
            rec = {'exception': h.args[0],'type': 'HTTPError','message': h.args[1]}
            response_data = func(rec)
        except Exception as e:
            response.status = 500
            expires=0
            rec = {'exception': 500, 'type': e.__class__.__name__, 'message': 'Server Error'}
            response_data = func(rec)
        _setCherryPyHeaders(response_data, datatype, expires)
        return response_data



#!/usr/bin/env python
"""
_SetLocationForWorkQueue_

MySQL implementation of Files.SetLocationForWorkQueue

For WorkQueue only
"""

from WMCore.Database.DBFormatter import DBFormatter


class SetLocationForWorkQueue(DBFormatter):
    """
    _SetLocationForWorkQueue_

    Set the location for a file, deleting all previous references
    and attaching the current references
    """

    deleteSQL = """DELETE FROM wmbs_file_location
                     WHERE fileid = (SELECT wfd.id FROM wmbs_file_details wfd WHERE wfd.lfn = :lfn)"""

    insertSQL = """INSERT IGNORE INTO wmbs_file_location (fileid, pnn)
                     SELECT wfd.id, wpnn.id
                     FROM wmbs_pnns wpnn, wmbs_file_details wfd
                     WHERE wpnn.pnn = :location
                     AND wfd.lfn = :lfn"""

    def execute(self, lfns, locations, isDBS=True, conn=None, transaction=None):
        """
        If files are from DBS:
        First, delete all file_location references with that lfn.
        Then, insert the new ones.

        Else:
        Simply insert the new locations
        """
        if isDBS:
            binds = []
            for lfn in lfns:
                binds.append({'lfn': lfn})
            self.dbi.processData(self.deleteSQL, binds, conn=conn,
                                 transaction=transaction)

        self.dbi.processData(self.insertSQL, locations, conn=conn,
                             transaction=transaction)

        return



# -*- coding: utf-8 -*-
"""
_SetRunningSlots_

MySQL implementation of Locations.SetRunningSlots

Created on Mon Jun 18 13:18:57 2012

@author: dballest
"""

from WMCore.Database.DBFormatter import DBFormatter

class SetRunningSlots(DBFormatter):
    sql = """UPDATE wmbs_location
             SET running_slots = :slots
             WHERE site_name = :location
             """

    def execute(self, siteName, jobSlots = 0, conn = None, transaction = False):
        binds = {"location": siteName, "slots": jobSlots}
        self.dbi.processData(self.sql, binds, conn = conn,
                             transaction = transaction)
        return



#!/usr/bin/env python
"""
_New_

MySQL implementation of Masks.New
"""

from WMCore.Database.DBFormatter import DBFormatter


class New(DBFormatter):
    sql = """INSERT INTO wmbs_job_mask (job, firstevent, lastevent, firstrun, lastrun, firstlumi, lastlumi, inclusivemask) VALUES
               (:jobid, :firstevent, :lastevent, :firstrun, :lastrun, :firstlumi, :lastlumi, :inclusivemask)"""

    def getDictBinds(self, jobList, inclusivemask):
        binds = []
        for job in jobList:
            binds.append({'jobid': job['id'], 'inclusivemask': inclusivemask,
                          'firstevent': job['mask']['FirstEvent'],
                          'lastevent': job['mask']['LastEvent'],
                          'firstrun': job['mask']['FirstRun'],
                          'lastrun': job['mask']['LastRun'],
                          'firstlumi': job['mask']['FirstLumi'],
                          'lastlumi': job['mask']['LastLumi'], })

        return binds

    def execute(self, jobList, inclusivemask=True, conn=None, transaction=False):
        binds = self.getDictBinds(jobList, inclusivemask)
        self.dbi.processData(self.sql, binds, conn=conn, transaction=transaction)
        return



#!/usr/bin/env python
"""
_BulkNewReturn_

Oracle implementation of JobGroup.BulkNewReturn
"""




from WMCore.WMBS.MySQL.JobGroup.BulkNewReturn import BulkNewReturn as MySQLBulkNewReturn

class BulkNewReturn(MySQLBulkNewReturn):
    """
    Does a bulk commit of jobGroups, followed by returning their IDs and UIDs

    """

    sql = """INSERT INTO wmbs_jobgroup (subscription, guid, output, last_update)
             VALUES (:subscription, :guid, :output, :timestamp)"""



"""
_InsertComponent_

MySQL implementation of UpdateWorker
"""

__all__ = []

import time

from WMCore.Database.DBFormatter import DBFormatter


class UpdateWorker(DBFormatter):
    sqlpart1 = """UPDATE wm_workers
                      SET last_updated = :last_updated
               """
    sqlpart3 = """ WHERE name = :worker_name"""

    def execute(self, workerName, state=None, timeSpent=None,
                results=None, conn=None, transaction=False):

        binds = {"worker_name": workerName,
                 "last_updated": int(time.time())}

        sqlpart2 = ""
        if state:
            binds["state"] = state
            sqlpart2 += ", state = :state"
        if timeSpent is not None:
            binds["cycle_time"] = timeSpent
            sqlpart2 += ", cycle_time = :cycle_time"
            binds["outcome"] = results
            sqlpart2 += ", outcome = :outcome"

        sql = self.sqlpart1 + sqlpart2 + self.sqlpart3

        self.dbi.processData(sql, binds, conn=conn,
                             transaction=transaction)
        return



#!/usr/bin/env python
"""
_LoadFilesByBlock_

MySQL implementation of LoadFilesByBlock
"""

from WMComponent.DBS3Buffer.MySQL.LoadDBSFilesByDAS import LoadDBSFilesByDAS as MySQLLoadDBSFilesByDAS

class LoadFilesByBlock(MySQLLoadDBSFilesByDAS):
    fileInfoSQL = """SELECT files.id AS id, files.lfn AS lfn, files.filesize AS filesize,
                    files.events AS events,
                    files.status AS status,
                    dbsbuffer_workflow.name AS workflow,
                    dbsbuffer_algo.app_name AS app_name, dbsbuffer_algo.app_ver AS app_ver,
                    dbsbuffer_algo.app_fam AS app_fam, dbsbuffer_algo.pset_hash AS pset_hash,
                    dbsbuffer_algo.config_content, dbsbuffer_dataset.path AS dataset_path,
                    dbsbuffer_dataset.acquisition_era AS acquisition_era,
                    dbsbuffer_dataset.processing_ver AS processing_ver,
                    dbsbuffer_dataset.global_tag AS global_tag,
                    dbsbuffer_dataset.prep_id AS prep_id,
                    dbsbuffer_workflow.block_close_max_wait_time,
                    dbsbuffer_workflow.block_close_max_files,
                    dbsbuffer_workflow.block_close_max_events,
                    dbsbuffer_workflow.block_close_max_size
             FROM dbsbuffer_file files
             INNER JOIN dbsbuffer_algo_dataset_assoc ON
               files.dataset_algo = dbsbuffer_algo_dataset_assoc.id
             INNER JOIN dbsbuffer_algo ON
               dbsbuffer_algo_dataset_assoc.algo_id = dbsbuffer_algo.id
             INNER JOIN dbsbuffer_dataset ON
               dbsbuffer_algo_dataset_assoc.dataset_id = dbsbuffer_dataset.id
             INNER JOIN dbsbuffer_workflow ON
               dbsbuffer_workflow.id = files.workflow
             WHERE files.block_id = (SELECT id FROM dbsbuffer_block WHERE blockname = :block)
             ORDER BY files.id"""

    def execute(self, blockname, conn = None, transaction = False):
        """
        This loads all files in a block.

        It has no defenses against loading files with unloaded parents.
        It depends on nobody but LoadDBSFilesByDAS putting files into a block
        """
        result   = self.dbi.processData(self.fileInfoSQL, {'block': blockname},
                                        conn = conn,
                                        transaction = transaction)
        fileInfo = self.formatFileInfo(result)

        fileIDs  = [x['id'] for x in fileInfo]
        binds    = self.getBinds(fileIDs)

        if len(fileInfo) == 0:
            # Then we have no files for this DAS
            return []


        # Do locations
        result   = self.dbi.processData(self.getLocationSQL, binds,
                                        conn = conn,
                                        transaction = transaction)
        locInfo  = self.locInfo(result)
        fullResults = self.merge(fileInfo, locInfo)


        # Do checksums
        result   = self.dbi.processData(self.getChecksumSQL, binds,
                                        conn = conn,
                                        transaction = transaction)

        ckInfo      = self.ckInfo(result)
        fullResults = self.merge(fullResults, ckInfo)


        # Do runLumi
        result      = self.dbi.processData(self.getRunLumiSQL, binds,
                                           conn = conn,
                                           transaction = transaction)
        runInfo  = self.runInfo(result)
        fullResults = self.merge(fullResults, runInfo)



        # Do parents
        result   = self.dbi.processData(self.getParentLFNSQL, binds,
                                        conn = conn,
                                        transaction = transaction)
        parInfo  = self.parentInfo(result)
        fullResults = self.merge(fullResults, parInfo)



        return fullResults
